# What is siuje
It is a very very tiny web server library, for Node.js.

# How to use
## Startup the server
```javascript
// index.js
const siuje = require("siuje");
siuje.work({ host: "127.0.0.1", port: 3000 })
```
## Schedule a job
```javascript
// index.js
const siuje = require("siuje");
siuje.when( "12:00" ).work( 
	async () => {
		// do some work here
	}
);
siuje.work({ host: "127.0.0.1", port: 3000 })
```
or
```javascript
// index.js
const siuje = require("siuje");
const {second, minute, hour, day} = require("siuje/utime");
siuje.every( second(10) ).repeat(
	async () => {
		// do something
	}
)//.until( () => true|false );
siuje.work({ host: "127.0.0.1", port: 3000 })
```
## Handle a http request
```javascript
// index.js
const siuje = require("siuje");

// sample for middleware
const set_header = headers => {
	return (req, ctrl) => next => {
		ctrl( headers );
		return next(req, ctrl);
	}
}

siuje.meet("/test")
	.use( set_header({"Server": "siuje"}) )
	.serve( async (req, ctrl) => {
		return "hello, world";
	});

siuje.work({ host: "127.0.0.1", port: 3000 })
```
# How about bodyParser?
You should implement your middleware for body parser.
# Detail for req/ctrl and next
- req is the instance for http.IncomingMessage
- next is only a function to run the next middleware
- ctrl is a handler for building the response
    1. when you give a number to ctrl, it means that you want to set the http response status code
    2. when you give an object to ctrl, it means that you want to set the http response header
    3. when you give a string(json/query/else) to ctrl, it means that you want to set the http response mine type
    4. when you give an empty to ctrl, it means that you want to get the current status/header and mine type
    5. when you give an error object to ctrl, it will give a 500 for response
    6. when you give a function to ctrl, your function will be call with the http response object and a setter to custom your output
